@@ -1,17 +1,23 @@
+from typing import Any, Optional, Tuple
+
 from sqlvalidator.grammar.sql import (
     Alias,
     AnalyticsClause,
     ArithmaticOperator,
+    Array,
+    ArrayAggFunctionCall,
     BitwiseOperation,
     Boolean,
     BooleanCondition,
     Case,
     CastFunctionCall,
+    ChainedColumns,
     Column,
     CombinedQueries,
     Condition,
     DatePartExtraction,
     ExceptClause,
+    Expression,
     FunctionCall,
     GroupByClause,
     HavingClause,
@@ -98,10 +104,7 @@ class SelectStatementParser:
             from_statement = None
 
         if lower(next_token) == "where":
-            expression_tokens, next_token = get_tokens_until_one_of(
-                tokens, ["group", "having", "order", "limit", "offset", ";"]
-            )
-            where_clause = WhereClauseParser.parse(iter(expression_tokens))
+            where_clause, next_token = WhereClauseParser.parse(tokens)
         else:
             where_clause = None
 
@@ -117,10 +120,7 @@ class SelectStatementParser:
             group_by_clause = None
 
         if lower(next_token) == "having":
-            expression_tokens, next_token = get_tokens_until_one_of(
-                tokens, ["order", "limit", "offset", ";"]
-            )
-            having_clause = HavingClauseParser.parse(iter(expression_tokens))
+            having_clause, next_token = HavingClauseParser.parse(tokens)
         else:
             having_clause = None
 
@@ -177,15 +177,30 @@ class FromStatementParser:
             except ParsingError:
                 argument = FromStatementParser.parse(iter(argument_tokens))
             expression = Parenthesis(argument)
+            next_token = next(tokens, None)
         elif next_token == "'" or next_token == '"' or next_token == "`":
             expression = Table(StringParser.parse(tokens, next_token))
+            next_token = next(tokens, None)
+        elif next_token == "[":
+            argument_tokens, next_token = get_tokens_until_one_of(
+                tokens, stop_words=["]"]
+            )
+            table, _ = ExpressionParser.parse(iter(argument_tokens))
+            expression = Table(table, in_square_brackets=True)
+            assert next_token == "]", next_token
+            next_token = next(tokens, None)
         else:
             if lower(next_token) == "unnest":
                 expression = UnnestParser.parse(tokens)
+                next_token = next(tokens, None)
             else:
-                expression = Table(next_token)
+                argument_tokens, next_token = get_tokens_until_one_of(
+                    tokens,
+                    stop_words=Join.VALUES + CombinedQueries.SET_OPERATORS,
+                    first_token=next_token,
+                )
+                expression = Table(ExpressionParser.parse(iter(argument_tokens))[0])
 
-        next_token = next(tokens, None)
         if (
             next_token is not None
             and lower(next_token) not in Join.VALUES
@@ -236,9 +251,13 @@ class FromStatementParser:
                 tokens, Join.VALUES, first_token=next_token
             )
             join_type = JoinTypeParser.parse(iter(expression_tokens))
-            expression_tokens, next_token = get_tokens_until_one_of(
-                tokens, ("on", "using"), first_token=next_token
-            )
+
+            if join_type in ("CROSS JOIN", ","):
+                expression_tokens, next_token = [next_token] + list(tokens), None
+            else:
+                expression_tokens, next_token = get_tokens_until_one_of(
+                    tokens, ("on", "using"), first_token=next_token
+                )
             right_expr = FromStatementParser.parse(iter(expression_tokens))
             on = None
             using = None
@@ -248,10 +267,10 @@ class FromStatementParser:
                 Join.VALUES,
             )
             if lower(on_or_using) == "on":
-                expression = ExpressionParser.parse(iter(expression_tokens))
+                expression, _ = ExpressionParser.parse(iter(expression_tokens))
                 on = OnClause(expression)
             elif lower(on_or_using) == "using":
-                expressions = ExpressionParser.parse(iter(expression_tokens))
+                expressions, _ = ExpressionParser.parse(iter(expression_tokens))
                 using = UsingClause(expressions)
             elif join_type not in ("CROSS JOIN", ","):
                 raise ParsingError("Missing ON or USING for join")
@@ -341,8 +360,12 @@ class SetOperatorTypeParser:
 class WhereClauseParser:
     @staticmethod
     def parse(tokens):
-        expression = ExpressionParser.parse(tokens)
-        return WhereClause(expression)
+        expression, next_token = ExpressionParser.parse(
+            tokens,
+            can_alias=False,
+            until_one_of=("group", "having", "order", "limit", "offset", ";"),
+        )
+        return WhereClause(expression), next_token
 
 
 class GroupByParser:
@@ -364,8 +387,10 @@ class GroupByParser:
 class HavingClauseParser:
     @staticmethod
     def parse(tokens):
-        expression = ExpressionParser.parse(tokens)
-        return HavingClause(expression)
+        expression, next_token = ExpressionParser.parse(
+            tokens, can_alias=False, until_one_of=["order", "limit", "offset", ";"]
+        )
+        return HavingClause(expression), next_token
 
 
 class OrderByParser:
@@ -388,7 +413,7 @@ class OrderByItemParser:
     @staticmethod
     def parse(tokens):
         expression_tokens, next_token = get_tokens_until_one_of(tokens, ["asc", "desc"])
-        expression = ExpressionParser.parse(iter(expression_tokens))
+        expression, _ = ExpressionParser.parse(iter(expression_tokens))
         if lower(next_token) == "asc":
             has_asc = True
             has_desc = False
@@ -414,14 +439,14 @@ class LimitClauseParser:
             expression_tokens, next_token = get_tokens_until_one_of(
                 tokens, [], first_token=next_token
             )
-            expression = ExpressionParser.parse(iter(expression_tokens))
+            expression, _ = ExpressionParser.parse(iter(expression_tokens))
         return LimitClause(limit_all, expression)
 
 
 class OffsetClauseParser:
     @staticmethod
     def parse(tokens):
-        expression = ExpressionParser.parse(tokens)
+        expression, _ = ExpressionParser.parse(tokens)
         return OffsetClause(expression)
 
 
@@ -433,19 +458,28 @@ class ExpressionListParser:
         expressions = []
         expression_tokens = []
         count_parenthesis = 0
+        count_square_brackets = 0
         while next_token is not None:
             expression_tokens.append(next_token)
             if next_token == "(":
                 count_parenthesis += 1
             elif next_token == ")":
                 count_parenthesis -= 1
+            elif next_token == "[":
+                count_square_brackets += 1
+            elif next_token == "]":
+                count_square_brackets -= 1
+
             next_token = next(tokens, None)
-            if next_token is None or (next_token == "," and count_parenthesis == 0):
-                expressions.append(
-                    ExpressionParser.parse(
-                        iter(expression_tokens), can_be_type=can_be_type
-                    )
+            if next_token is None or (
+                next_token == ","
+                and count_parenthesis == 0
+                and count_square_brackets == 0
+            ):
+                expression, _ = ExpressionParser.parse(
+                    iter(expression_tokens), can_be_type=can_be_type
                 )
+                expressions.append(expression)
                 expression_tokens = []
                 next_token = next(tokens, None)
         return expressions
@@ -453,7 +487,15 @@ class ExpressionListParser:
 
 class ExpressionParser:
     @staticmethod
-    def parse(tokens, is_right_hand=False, can_be_type=False):
+    def parse(
+        tokens,
+        is_right_hand=False,
+        can_be_type=False,
+        can_alias=True,
+        until_one_of=None,
+    ) -> Tuple[Expression, Any]:
+        until_one_of = until_one_of or []
+
         main_token = next(tokens)
         next_token = None
 
@@ -467,13 +509,23 @@ class ExpressionParser:
             expression = Null()
         elif lower(main_token) == Negation.PREDICATE:
             rest_expression, next_token = ExpressionParser.parse(
-                tokens, is_right_hand=True
+                tokens,
+                is_right_hand=True,
+                until_one_of=until_one_of,
             )
             expression = Negation(rest_expression)
         elif main_token == "(":
             argument_tokens = get_tokens_until_closing_parenthesis(tokens)
             arguments = ExpressionListParser.parse(iter(argument_tokens))
             expression = Parenthesis(*arguments)
+        elif main_token == "[":
+            argument_tokens, next_token = get_tokens_until_one_of(
+                tokens, stop_words=["]"]
+            )
+            assert next_token == "]", next_token
+            arguments = ExpressionListParser.parse(iter(argument_tokens))
+            expression = Array(*arguments)
+            next_token = next(tokens, None)
         elif lower(main_token) == "case":
             argument_tokens, next_token = get_tokens_until_one_of(tokens, ["end"])
             assert lower(next_token) == "end"
@@ -497,7 +549,9 @@ class ExpressionParser:
                         tokens, stop_words=["as"]
                     )
                     column, _ = ExpressionParser.parse(
-                        iter(column_tokens), is_right_hand=True
+                        iter(column_tokens),
+                        is_right_hand=True,
+                        until_one_of=until_one_of,
                     )
                     assert lower(next_token) == "as", next_token
                     next_token = next(tokens)
@@ -505,6 +559,61 @@ class ExpressionParser:
                     expression = CastFunctionCall(column, cast_type)
                     next_token = next(tokens)
                     assert lower(next_token) == ")", next_token
+                elif lower(main_token) == "array_agg":
+                    next_token = next(tokens)
+                    if lower(next_token) == "distinct":
+                        distinct = True
+                        first_token = None
+                    else:
+                        distinct = False
+                        first_token = next_token
+
+                    column_tokens, next_token = get_tokens_until_one_of(
+                        tokens,
+                        stop_words=[")", "ignore", "respects", "order", "limit"],
+                        first_token=first_token,
+                    )
+                    column, _ = ExpressionParser.parse(
+                        iter(column_tokens), until_one_of=until_one_of
+                    )
+
+                    ignore_nulls = respect_nulls = False
+                    if lower(next_token) == "ignore":
+                        next_token = next(tokens)
+                        assert lower(next_token) == "nulls"
+                        ignore_nulls = True
+                        next_token = next(tokens)
+                    elif lower(next_token) == "respect":
+                        next_token = next(tokens)
+                        assert lower(next_token) == "nulls"
+                        respect_nulls = True
+                        next_token = next(tokens)
+
+                    if lower(next_token) == "order":
+                        next_token = next(tokens)
+                        assert lower(next_token) == "by"
+                        expression_tokens, next_token = get_tokens_until_one_of(
+                            tokens, ["limit", ")"]
+                        )
+                        order_bys = OrderByParser.parse(iter(expression_tokens))
+                    else:
+                        order_bys = None
+
+                    limit = None
+                    if lower(next_token) == "limit":
+                        next_token = next(tokens)
+                        limit = int(next_token)
+                        next_token = next(tokens)
+
+                    assert lower(next_token) == ")", next_token
+                    expression = ArrayAggFunctionCall(
+                        column=column,
+                        distinct=distinct,
+                        ignore_nulls=ignore_nulls,
+                        respect_nulls=respect_nulls,
+                        order_bys=order_bys,
+                        limit=limit,
+                    )
                 else:
                     argument_tokens = get_tokens_until_closing_parenthesis(tokens)
                     arguments_can_be_type = (
@@ -520,9 +629,10 @@ class ExpressionParser:
                 and lower(main_token) in DatePartExtraction.PARTS
                 and lower(next_token) == "from"
             ):
-                rest_expression = ExpressionParser.parse(tokens)
+                rest_expression, next_token = ExpressionParser.parse(
+                    tokens, until_one_of=until_one_of
+                )
                 expression = DatePartExtraction(main_token, rest_expression)
-                next_token = next(tokens, None)
             elif lower(main_token) in Type.VALUES and can_be_type:
                 expression = Type(main_token)
             elif next_token is not None and next_token == "[":
@@ -580,7 +690,7 @@ class ExpressionParser:
             if lower(argument_next_token) in ("rows", "range"):
                 rows_range = argument_next_token
                 expression_tokens, _ = get_tokens_until_one_of(argument_tokens, [])
-                frame_clause = WindowFrameClause(
+                frame_clause: Optional[WindowFrameClause] = WindowFrameClause(
                     rows_range, " ".join(expression_tokens)
                 )
             else:
@@ -597,9 +707,17 @@ class ExpressionParser:
         if next_token and next_token in ("+", "-", "*", "/"):
             left_hand = expression
             symbol = next_token
-            right_hand, next_token = ExpressionParser.parse(tokens, is_right_hand=True)
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, is_right_hand=True, until_one_of=until_one_of
+            )
             expression = ArithmaticOperator(symbol, left_hand, right_hand)
 
+        if next_token == ".":
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, is_right_hand=True, until_one_of=until_one_of
+            )
+            expression = ChainedColumns(expression, right_hand)
+
         if is_right_hand:
             return expression, next_token
 
@@ -615,17 +733,19 @@ class ExpressionParser:
                     )
                     tokens = iter(tokens)
 
-            right_hand, next_token = ExpressionParser.parse(tokens, is_right_hand=True)
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, is_right_hand=True, until_one_of=until_one_of
+            )
             expression = Condition(expression, symbol, right_hand)
         elif lower(next_token) == "between":
             symbol = next_token
             right_hand_left, next_token = ExpressionParser.parse(
-                tokens, is_right_hand=True
+                tokens, is_right_hand=True, until_one_of=until_one_of
             )
             if lower(next_token) != "and":
                 raise ParsingError("expected AND")
             right_hand_right, next_token = ExpressionParser.parse(
-                tokens, is_right_hand=True
+                tokens, is_right_hand=True, until_one_of=until_one_of
             )
             right_hand = BooleanCondition(
                 "and",
@@ -635,15 +755,18 @@ class ExpressionParser:
             expression = Condition(expression, symbol, right_hand)
         elif next_token in BitwiseOperation.OPERATORS:
             operator = next_token
-            right_hand, next_token = ExpressionParser.parse(tokens, is_right_hand=True)
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, is_right_hand=True, until_one_of=until_one_of
+            )
             expression = BitwiseOperation(expression, operator, right_hand)
 
         if lower(next_token) in BooleanCondition.PREDICATES:
             left_hand = expression
             symbol = next_token
-            right_hand = ExpressionParser.parse(tokens)
+            right_hand, next_token = ExpressionParser.parse(
+                tokens, until_one_of=until_one_of
+            )
             expression = BooleanCondition(symbol, left_hand, right_hand)
-            next_token = next(tokens, None)
 
         if lower(next_token) == "except":
             opening_parenthesis = next(tokens, None)
@@ -661,15 +784,19 @@ class ExpressionParser:
             and next_token != '"'
             and next_token != "`"
             and next_token != ";"
+            and lower(next_token) not in until_one_of
+            and can_alias
         ):
             if lower(next_token) == "as":
                 with_as = True
-                alias, _ = ExpressionParser.parse(tokens, is_right_hand=True)
+                alias, _ = ExpressionParser.parse(
+                    tokens, is_right_hand=True, until_one_of=until_one_of
+                )
             else:
                 with_as = False
                 alias = next_token
-            return Alias(expression, alias, with_as)
-        return expression
+            return Alias(expression, alias, with_as), next(tokens, None)
+        return expression, next_token
 
 
 class StringParser:
@@ -698,21 +825,21 @@ class CaseParser:
             expressions_tokens, next_token = get_tokens_until_one_of(
                 tokens, ["when"], first_token=next_token
             )
-            expression = ExpressionParser.parse(iter(expressions_tokens))
+            expression, _ = ExpressionParser.parse(iter(expressions_tokens))
 
         when_then = []
         else_expression = None
         while next_token:
             if lower(next_token) == "when":
                 expressions_tokens, _ = get_tokens_until_one_of(tokens, ["then"])
-                when_expression = ExpressionParser.parse(iter(expressions_tokens))
+                when_expression, _ = ExpressionParser.parse(iter(expressions_tokens))
                 expressions_tokens, next_token = get_tokens_until_one_of(
                     tokens, ["when", "else"]
                 )
-                then_expression = ExpressionParser.parse(iter(expressions_tokens))
+                then_expression, _ = ExpressionParser.parse(iter(expressions_tokens))
                 when_then.append((when_expression, then_expression))
             elif lower(next_token) == "else":
                 expressions_tokens, next_token = get_tokens_until_one_of(tokens, [])
-                else_expression = ExpressionParser.parse(iter(expressions_tokens))
+                else_expression, _ = ExpressionParser.parse(iter(expressions_tokens))
 
         return Case(expression, when_then, else_expression)
