@@ -72,9 +72,13 @@ class SelectStatement:
                 from_statement = self.from_statement
 
             if isinstance(from_statement, Parenthesis):
-                from_str = "(\n{}\n)".format(
-                    from_statement.args[0].transform(is_subquery=True)
-                )
+                if isinstance(from_statement.args[0], SelectStatement):
+                    inner_from_str = from_statement.args[0].transform(is_subquery=True)
+                else:
+                    inner_from_str = " " + transform(from_statement.args[0]).replace(
+                        "\n", "\n "
+                    )
+                from_str = f"(\n{inner_from_str}\n)"
             else:
                 from_str = transform(from_statement)
 
@@ -319,10 +323,13 @@ class OrderByClause(Expression):
     def __init__(self, *args):
         self.args = args
 
-    def __str__(self):
+    def transform(self, allow_linebreak=True):
         if len(self.args) > 1:
-            order_by_str = "\n" + ",\n".join(map(str, self.args))
-            order_by_str = order_by_str.replace("\n", "\n ")
+            if allow_linebreak:
+                order_by_str = "\n" + ",\n".join(map(str, self.args))
+                order_by_str = order_by_str.replace("\n", "\n ")
+            else:
+                order_by_str = " " + ", ".join(map(str, self.args))
         else:
             order_by_str = " " + transform(self.args[0])
         return order_by_str
@@ -482,6 +489,47 @@ class CastFunctionCall(FunctionCall):
         )
 
 
+class ArrayAggFunctionCall(FunctionCall):
+    def __init__(
+        self,
+        column,
+        distinct=False,
+        ignore_nulls=False,
+        respect_nulls=False,
+        order_bys=None,
+        limit=None,
+    ):
+        super().__init__("array_agg", column)
+        self.distinct = distinct
+        assert not (ignore_nulls and respect_nulls)
+        self.ignore_nulls = ignore_nulls
+        self.respect_nulls = respect_nulls
+        self.order_bys = order_bys
+        self.limit = limit
+
+    def __str__(self):
+        array_agg_str = "{}(".format(self.function_name.upper())
+        if self.distinct:
+            array_agg_str += "DISTINCT "
+
+        array_agg_str += transform(self.args[0]) + " "
+
+        if self.ignore_nulls:
+            array_agg_str += "IGNORE NULLS "
+        elif self.respect_nulls:
+            array_agg_str += "RESPECT NULLS "
+
+        if self.order_bys:
+            array_agg_str += "ORDER BY{}".format(
+                self.order_bys.transform(allow_linebreak=False)
+            )
+
+        if self.limit:
+            array_agg_str += " LIMIT {}".format(self.limit)
+
+        return array_agg_str + ")"
+
+
 class AnalyticsClause(Expression):
     def __init__(self, function, partition_by, order_by, frame_clause):
         self.function = function
@@ -598,6 +646,24 @@ class Column(Expression):
         return errors
 
 
+class ChainedColumns(Expression):
+    def __init__(self, *args):
+        self.columns = args
+
+    def __str__(self):
+        return ".".join(map(transform, self.columns))
+
+    def __repr__(self):
+        return "<ChainedColumns: {}>".format(".".join(map(repr, self.columns)))
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and len(self.columns) == len(other.columns)
+            and all(a == o for a, o in zip(self.columns, other.columns))
+        )
+
+
 class Type(Expression):
     VALUES = ("int", "float", "day", "month", "timestamp", "int64", "string", "date")
 
@@ -760,6 +826,24 @@ class Parenthesis(Expression):
             return a.return_type
 
 
+class Array(Expression):
+    def __init__(self, *args):
+        self.args = args
+
+    def __str__(self):
+        return "[{}]".format(", ".join(map(transform, self.args)))
+
+    def __repr__(self):
+        return "<Array: {}>".format(", ".join(map(repr, self.args)))
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and len(self.args) == len(other.args)
+            and all(a == o for a, o in zip(self.args, other.args))
+        )
+
+
 class Alias(Expression):
     def __init__(self, expression, alias, with_as):
         self.expression = expression
@@ -859,7 +943,15 @@ class BitwiseOperation(Expression):
 
 
 class Table(Expression):
-    pass
+    def __init__(self, value, in_square_brackets=False):
+        super().__init__(value)
+        self.in_square_brackets = in_square_brackets
+
+    def __str__(self):
+        table_str = transform(self.value)
+        if self.in_square_brackets:
+            table_str = f"[{table_str}]"
+        return table_str
 
 
 class Unnest(Expression):
@@ -1008,7 +1100,7 @@ class Join(Expression):
 
 
 class CombinedQueries(Expression):
-    SET_OPERATORS = ("union", "intersect", "except")
+    SET_OPERATORS = ("union", "intersect", "except", "all")
 
     def __init__(self, set_operator, left_query, right_query):
         self.set_operator = set_operator
